package l2vote;

import l2.gameserver.dao.CharacterDAO;
import l2.gameserver.instancemanager.ServerVariables;
import l2.gameserver.model.GameObjectsStorage;
import l2.gameserver.model.Player;
import l2.gameserver.taskmanager.DelayedItemsManager;

import java.sql.*;
import java.time.LocalDate;
import java.util.Random;

import static l2vote.Config.*;

public class DatabaseManager implements AutoCloseable {
    private Connection connection;

    public DatabaseManager(Connection connection) {
        this.connection = connection;
    }

    // Метод для создания таблицы, если она еще не существует
    public void createTableIfNotExists() throws SQLException {
        String query = "CREATE TABLE IF NOT EXISTS votes (" +
                "id INT NOT NULL, " + // ID из API (Не нужно PRIMARY KEY, если ID не автоинкремент)
                "char_name VARCHAR(255) NOT NULL, " +
                "vote_time DATETIME NOT NULL, " + // Время голосования из API
                "reward_given BOOLEAN DEFAULT FALSE, " + // Награда выдана
                "PRIMARY KEY(id)" + // Указание PRIMARY KEY для поля id
                ")";
        try (Statement stmt = connection.createStatement()) {
            stmt.execute(query);
        }
    }



    // Метод для добавления нового голоса в базу данных
    // Метод для добавления нового голоса
    public boolean addVote(String charName, String voteTime, int id) throws SQLException {
        String checkQuery = "SELECT 1 FROM votes WHERE id = ?";
        try (PreparedStatement ps = connection.prepareStatement(checkQuery)) {
            ps.setInt(1, id);
            ResultSet rs = ps.executeQuery();
            if (rs.next()) {
                return false; // Голос с таким ID уже существует
            }
        }

        // Добавляем запись в таблицу голосования
        String insertQuery = "INSERT INTO votes (id, char_name, vote_time, reward_given) VALUES (?, ?, ?, FALSE)";
        try (PreparedStatement ps = connection.prepareStatement(insertQuery)) {
            ps.setInt(1, id); // ID уникален, будет использоваться для проверки
            ps.setString(2, charName);
            ps.setString(3, voteTime);
            ps.executeUpdate();
        }
        return true;
    }

    public void giveRewards(boolean allowMultipleRewards) throws SQLException {
        String today = java.time.LocalDate.now().toString(); // Текущая дата в формате yyyy-MM-dd

        // Выборка голосов с учетом allowMultipleRewards
// Выборка голосов с учетом allowMultipleRewards
        String selectQuery = allowMultipleRewards
                ? "SELECT char_name, id FROM votes WHERE reward_given = FALSE AND DATE(vote_time) = ?"
                : "SELECT char_name, id FROM votes WHERE reward_given = FALSE AND DATE(vote_time) = ?";  // исправлено

        try (PreparedStatement ps = connection.prepareStatement(selectQuery)) {
            ps.setString(1, today); // Устанавливаем текущую дату
            ResultSet rs = ps.executeQuery();

            while (rs.next()) {
                String charName = rs.getString("char_name");
                int voteId = rs.getInt("id");

                // Проверяем, существует ли игрок с таким ником за сегодняшний день
                String checkPlayerQuery = "SELECT COUNT(*) FROM votes WHERE char_name = ? AND DATE(vote_time) = ?";
                try (PreparedStatement checkPlayerPs = connection.prepareStatement(checkPlayerQuery)) {
                    checkPlayerPs.setString(1, charName);
                    checkPlayerPs.setString(2, today);
                    ResultSet checkPlayerRs = checkPlayerPs.executeQuery();

                    if (checkPlayerRs.next() && checkPlayerRs.getInt(1) == 0) {
                        // Если игрока с таким ником нет в базе данных за сегодня, добавляем его
                        String insertQuery = "INSERT INTO votes (char_name, vote_time, reward_given) VALUES (?, ?, FALSE)";
                        try (PreparedStatement insertPs = connection.prepareStatement(insertQuery)) {
                            insertPs.setString(1, charName);
                            insertPs.setString(2, today);
                            insertPs.executeUpdate();
                        }

                        // Выдаем награду, так как игрока нет в базе данных
                        if(CheckIpAndHwid(charName)) {
                            grantReward(charName);
                        }else {
                            int objId = CharacterDAO.getInstance().getObjectIdByName(charName);
                            if(objId > 0 && GameObjectsStorage.getPlayer(objId) != null){
                                Player player = GameObjectsStorage.getPlayer(objId);
                                player.sendMessage("Вы уже сегодня голосовали на этот IP или HWID, попробуйте после 00:00");
                            }
                        }
                        markRewardAsGiven(charName, today);
                    } else {
                        // Если игрок есть в базе, проверяем, была ли выдана награда
                        if (!allowMultipleRewards) {
                            String checkRewardQuery = "SELECT COUNT(*) FROM votes WHERE char_name = ? AND DATE(vote_time) = ? AND reward_given = TRUE";
                            try (PreparedStatement checkPs = connection.prepareStatement(checkRewardQuery)) {
                                checkPs.setString(1, charName);
                                checkPs.setString(2, today);
                                ResultSet checkRs = checkPs.executeQuery();

                                if (checkRs.next() && checkRs.getInt(1) == 0) {
                                    // Если награда еще не выдана, выдаем награду
                                    if(CheckIpAndHwid(charName)) {
                                        grantReward(charName);
                                    }else {
                                        int objId = CharacterDAO.getInstance().getObjectIdByName(charName);
                                        if(objId > 0 && GameObjectsStorage.getPlayer(objId) != null){
                                            Player player = GameObjectsStorage.getPlayer(objId);
                                            player.sendMessage("Вы уже сегодня голосовали на этот IP или HWID, попробуйте после 00:00");
                                        }
                                    }
                                    markRewardAsGiven(charName, today);
                                }else{
                                    int objId = CharacterDAO.getInstance().getObjectIdByName(charName);
                                    if(objId > 0 && GameObjectsStorage.getPlayer(objId) != null){
                                        Player player = GameObjectsStorage.getPlayer(objId);
                                        player.sendMessage("Вы уже сегодня голосовали на этот ник, попробуйте после 00:00");
                                    }
                                }
                            }
                        } else {
                            // Если allowMultipleRewards == true, проверяем, был ли уже обработан голос с таким ID
                            String checkVoteQuery = "SELECT COUNT(*) FROM votes WHERE id = ? AND reward_given = TRUE";
                            try (PreparedStatement checkVotePs = connection.prepareStatement(checkVoteQuery)) {
                                checkVotePs.setInt(1, voteId);
                                ResultSet checkVoteRs = checkVotePs.executeQuery();

                                if (checkVoteRs.next() && checkVoteRs.getInt(1) == 0) {
                                    // Если награда за этот голос еще не выдана, выдаем награду
                                    if(CheckIpAndHwid(charName)) {
                                        grantReward(charName);
                                    }else {
                                        int objId = CharacterDAO.getInstance().getObjectIdByName(charName);
                                        if(objId > 0 && GameObjectsStorage.getPlayer(objId) != null){
                                            Player player = GameObjectsStorage.getPlayer(objId);
                                            player.sendMessage("Вы уже сегодня голосовали на этот IP или HWID, попробуйте после 00:00");
                                        }
                                    }
                                    markRewardAsGiven(charName, today);
                                }
                            }
                        }
                    }
                }
            }
        }
    }


    public boolean CheckIpAndHwid(String charName) throws SQLException {
        String ip = null;
        String hwid = null;

        String getAccountNameQuery = "SELECT account_name FROM characters WHERE char_name = ?";
        String getAccountDetailsQuery = "SELECT lastIP, lastHWID FROM accounts WHERE login = ?";

        if (connection == null || connection.isClosed()) {
            throw new IllegalStateException("Database connection is not initialized or closed.");
        }

        try {
            // Шаг 1: Получение account_name
            String accountName = null;
            try (PreparedStatement ps1 = connection.prepareStatement(getAccountNameQuery)) {
                ps1.setString(1, charName);
                try (ResultSet rs1 = ps1.executeQuery()) {
                    if (rs1.next()) {
                        accountName = rs1.getString("account_name");
                    }
                }
            }

            if (accountName == null) {
                System.err.println("Account name not found for character: " + charName);
                return false;
            }

            // Шаг 2: Получение lastIP и lastHWID
            try (PreparedStatement ps2 = connection.prepareStatement(getAccountDetailsQuery)) {
                ps2.setString(1, accountName);
                try (ResultSet rs2 = ps2.executeQuery()) {
                    if (rs2.next()) {
                        ip = rs2.getString("lastIP");
                        hwid = rs2.getString("lastHWID");
                    }
                }
            }

            // Шаг 3: Проверка IP и HWID
            if (ENABLE_CHECK_IP && ip != null) {
                String key = "l2vote_ip_" + ip;
                String storedDate = ServerVariables.getString(key,"null");
                if (storedDate != null && !storedDate.equals("null")) {
                    LocalDate localDate = LocalDate.parse(storedDate);
                    if (localDate.equals(LocalDate.now())) {
                        return false;
                    }
                }
                ServerVariables.set(key, LocalDate.now().toString());
            }

            if (ENABLE_CHECK_HWID && hwid != null) {
                String key = "l2vote_hwid_" + hwid;

                String storedDate = ServerVariables.getString(key,"null");
                if (storedDate != null && !storedDate.equals("null")) {
                    LocalDate localDate = LocalDate.parse(storedDate);
                    if (localDate.equals(LocalDate.now())) {
                        return false;
                    }
                }
                ServerVariables.set(key, LocalDate.now().toString());

            }

            return true;

        } catch (SQLException e) {
            e.printStackTrace();
            throw e; // Бросаем исключение для обработки на более высоком уровне
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }









    private void grantReward(String charName) {
        // Логика выдачи бонуса игроку
        int obj_id = CharacterDAO.getInstance().getObjectIdByName(charName);
        if(obj_id > 0) {
            if(!L2VoteRewardItemRandom) {
                for (String string : L2VoteRewardItemId) {
                    String[] str = string.split(":");
                    int id = Integer.parseInt(str[0]);
                    long count = Integer.parseInt(str[1]);
                    DelayedItemsManager.getInstance().addDelayed(obj_id, id, (int) count, 0, 0, 0, "L2vote");

                    LogManagerVote.getInstance().info("Выдача награды игроку (ID: "+id+", Name: "+charName+")" );
                }
            }else{
                Random random = new Random();
                int index = random.nextInt(L2VoteRewardItemId.length);
                String[] strings = L2VoteRewardItemId[index].split(":");
                int id = Integer.parseInt(strings[0]);
                long count = Integer.parseInt(strings[1]);
                DelayedItemsManager.getInstance().addDelayed(obj_id, id, (int) count, 0, 0, 0, "L2vote");
                //System.out.printf("Выдача награды игроку (ID: %d, Name: %s)%n", id, charName);
                LogManagerVote.getInstance().info("Выдача награды игроку (ID: "+id+", Name: "+charName+")" );
            }
        }else{
           // System.err.println("Ошибка при выдаче наград: Игрок с ником - "+charName +" не найден!");

            LogManagerVote.getInstance().error("Ошибка при выдаче наград: Игрок с ником - "+charName +" не найден!" );
        }
        LogManagerVote.getInstance().info("Выдан бонус игроку " + charName);

    }

    private void markRewardAsGiven(String charName, String today) throws SQLException {
        String updateQuery = "UPDATE votes SET reward_given = TRUE WHERE char_name = ? AND DATE(vote_time) = ?";
        try (PreparedStatement ps = connection.prepareStatement(updateQuery)) {
            ps.setString(1, charName);
            ps.setString(2, today); // Устанавливаем текущую дату
            ps.executeUpdate();
        }
    }

    @Override
    public void close() throws SQLException {
        if (connection != null) {
            connection.close();
        }
    }


    public void addReward(String charName){

    }


}
